'=========================================================================
' Prozeduren fr dreidimensionale Graphik
' 3D-GRAPH.BAS
' Autor: Lothar Schirm
' Letzte Ueberarbeitung 17.03.08
'=========================================================================

' Im Hauptprogramm muss vor dem Aufrufen der SUB Koordinaten3D, xyzGraphik, 
' zrphiGraphik oder zrGraphik der Graphikbildschirm mit den entsprechenden 
' mathematischen Fensterkoordinaten initialisiert sein, z.B.:
' SCREEN 12
' WINDOW (ymin, zmin)-(ymax, zmax)
' (Die Graphik wird auf die y-z-Ebene projiziert, siehe unten).


DECLARE SUB Projektion(x AS SINGLE, y AS SINGLE, z AS SINGLE, angp AS SINGLE, _
                       BYREF yp AS SINGLE, BYREF zp AS SINGLE)
DECLARE SUB Rotation(x AS SINGLE, y AS SINGLE, z AS SINGLE, Achse AS STRING,_
                     Winkel AS SINGLE, BYREF xrot AS SINGLE, _
                     BYREF yrot AS SINGLE, BYREF zrot AS SINGLE)
DECLARE SUB Koordinaten3D(xmin AS SINGLE, xmax AS SINGLE, ymin AS SINGLE, _
                          ymax AS SINGLE, zmin AS SINGLE, zmax AS SINGLE, _
                          gamma AS SINGLE, angp AS SINGLE, xAchse AS STRING, _
                          yAchse AS STRING, zAchse AS STRING, Text AS STRING)
DECLARE SUB xyzGraphik(x() AS SINGLE, y() AS SINGLE, z() AS SINGLE, _
                       nx AS INTEGER, ny AS INTEGER, gamma AS SINGLE, _
                       angp AS SINGLE, Farbe AS INTEGER, zref AS INTEGER)
DECLARE SUB zrphiGraphik(r() AS SINGLE, phi() AS SINGLE, z() AS SINGLE, _
                         nphi AS INTEGER, nz AS INTEGER, gamma AS SINGLE, _
                         angp AS SINGLE, Farbe AS INTEGER)
DECLARE SUB zrGraphik(r() AS SINGLE, z() AS SINGLE, n AS INTEGER, _
                      nphi AS INTEGER, angp AS SINGLE, Farbe AS INTEGER)


SUB Projektion(x AS SINGLE, y AS SINGLE, z AS SINGLE, angp AS SINGLE, _
               BYREF yp AS SINGLE, BYREF zp AS SINGLE)

  'Projektion eines Punktes x,y,z unter dem Winkel angp auf den
  'Bildschirm (y-z-Ebene, angp in Grad).
  'yp, zp sind die Koordinaten des projizierten Punktes.
  'Orientierung des Koordinatensystems:
  '
  '                 ^ z      
  '                 I
  '                 I
  '                 I
  '                 +----------> y
  '               /
  '             /
  '           x

  DIM AS SINGLE pi, phi
  
  pi = 3.141593
  phi = pi * angp / 180
  yp = y
  zp = z - x * TAN(phi)

END SUB


SUB Rotation(x AS SINGLE, y AS SINGLE, z AS SINGLE, Achse AS STRING,_
             Winkel AS SINGLE, BYREF xrot AS SINGLE, BYREF yrot AS SINGLE, _
             BYREF zrot AS SINGLE)

  'Berechnung der Koordinaten eines Punktes x,y,z nach Rotation
  'um eine Koordinatenachse.
  'Orientierung des Koordinatensystems wie bei SUB Projektion.
  'Parameterliste:
  '- x, y, z = Koordinaten vor der Rotation
  '- Achse$ = Achse, um die gedreht werden soll ("x", "y" oder "z")
  '- Winkel = Drehwinkel in Grad (entgegen dem Uhrzeigersinn)
  '- xrot, yrot, zrot = Koordinaten nach der Rotation

  DIM AS SINGLE pi, phi
  
  pi = 3.141593
  phi = pi * Winkel / 180     'Winkel im Bogenma
  SELECT CASE Achse
    CASE "x"    'Rotation um x-Achse:
        xrot = x
        yrot = y * COS(phi) - z * SIN(phi)
        zrot = y * SIN(phi) + z * COS(phi)
    CASE "y"    'Rotation um y-Achse:
        xrot = -x * COS(phi) - z * SIN(phi)
        yrot = y
        zrot = -x * SIN(phi) + z * COS(phi)
    CASE "z"    'Rotation um z-Achse:
        xrot = x * COS(phi) - y * SIN(phi)
        yrot = x * SIN(phi) + y * COS(phi)
        zrot = z
  END SELECT

END SUB


SUB Koordinaten3D(xmin AS SINGLE, xmax AS SINGLE, ymin AS SINGLE, _
                  ymax AS SINGLE, zmin AS SINGLE, zmax AS SINGLE, _
                  gamma AS SINGLE, angp AS SINGLE, xAchse AS STRING, _
                  yAchse AS STRING, zAchse AS STRING, Text AS STRING)
  
  '3D-Koordinatensystem in der Darstellung wie in der SUB Projektion, jedoch
  'zusaetzlich Drehung um die z-Achse moeglich.
  'Parameter:
  '- xmin ... zmax definieren die Abmessungen der x-, y- und z-Achse
  '- gamma = Drehwinkel um die z-Achse
  '- angp = Projektionswinkel
  '- xAchse, yAchse, zAchse = Achsenbeschriftungen
  '- Text = Bildunterschrift (Zeile 28, zentriert)
  'Bei Verwendung der Achsenbeschriftungen und der Bildunterschrift muss der
  'gesamte Bildschirm als Grafikfenster intitialisiert sein!
  'Die u.a. Graphiken (SUB xyzGraphik, oder zrphiGraphik) koennen in das
  'Koordinatensystem gezeichnet werden, wenn gamma und angp gleich gewaehlt
  'werden.
  
  DIM AS SINGLE yp_xmin, yp_xmax, yp_ymin, yp_ymax, yp_zmin, yp_zmax, _
                zp_xmin, zp_xmax, zp_ymin, zp_ymax, zp_zmin, zp_zmax
  DIM AS SINGLE xrot, yrot, zrot
  
  'Berechnung der Achsen-Endpunkte durch Rotation und Projektion:
   Rotation(xmin, 0, 0, "z", gamma, xrot, yrot, zrot)
   Projektion(xrot, yrot, zrot, angp, yp_xmin, zp_xmin)
   Rotation(xmax, 0, 0, "z", gamma, xrot, yrot, zrot)
   Projektion(xrot, yrot, zrot, angp, yp_xmax, zp_xmax)
   Rotation(0, ymin, 0, "z", gamma, xrot, yrot, zrot)
   Projektion(xrot, yrot, zrot, angp, yp_ymin, zp_ymin)
   Rotation(0, ymax, 0, "z", gamma, xrot, yrot, zrot)
   Projektion(xrot, yrot, zrot, angp, yp_ymax, zp_ymax)
   Rotation(0, 0, zmin, "z", gamma, xrot, yrot, zrot)
   Projektion(xrot, yrot, zrot, angp, yp_zmin, zp_zmin)
   Rotation(0, 0, zmax, "z", gamma, xrot, yrot, zrot)
   Projektion(xrot, yrot, zrot, angp, yp_zmax, zp_zmax)
  
  'Achsen zeichnen:
  LINE (yp_xmin, zp_xmin) - (yp_xmax, zp_xmax)
  LINE (yp_ymin, zp_ymin) - (yp_ymax, zp_ymax)
  LINE (yp_zmin, zp_zmin) - (yp_zmax, zp_zmax)
  
  DRAW STRING (yp_xmax, zp_xmax), xAchse
  DRAW STRING (yp_ymax, zp_ymax), yAchse
  DRAW STRING (yp_zmax, zp_zmax), zAchse
  LOCATE 28, (80 - LEN(Text)) / 2
  PRINT Text
  
END SUB


SUB xyzGraphik(x() AS SINGLE, y() AS SINGLE, z() AS SINGLE, nx AS INTEGER, _
               ny AS INTEGER, gamma AS SINGLE, angp AS SINGLE, _
               Farbe AS INTEGER, zref AS INTEGER)
  
  '3D-Graphik einer Funktion z(x, y).
  'Parameter:
  '- x von x(0) bis x(nx)
  '- y von y(0) bis y(ny)
  '- z von z(0,0) bis z(nx, ny)
  '- gamma = Drehwinkel um die z-Achse
  '- angp = Projektionswinkel
  '- Farbe = Farbe der gezeichneten Linien
  '- zref <> 0: Referenzebene z = 0 wird gezeichnet (empfiehlt sich, wenn mit
  '  der SUB Koordinaten3D ein Koordinatensystem gezeichnet wird).
      
  DIM AS SINGLE yp(nx, ny), zp(nx, ny)
  DIM AS SINGLE yp01, yp02, yp03, yp04, zp01, zp02, zp03, zp04
  DIM AS SINGLE xrot, yrot, zrot
  DIM AS INTEGER i, j
  
  'Berechnung der Funktionspunkte durch Rotation und Projektion:
  FOR i = 0 TO nx
    FOR j = 0 TO ny
      Rotation(x(i), y(j), z(i, j), "z", gamma, xrot, yrot, zrot)
      Projektion(xrot, yrot, zrot, angp, yp(i, j), zp(i, j))
    NEXT j
  NEXT i
  
  'Oberflaechenlinien der Funktion fuer x = const:
  FOR i = 0 TO nx
    FOR j = 1 TO ny
      LINE (yp(i, j-1), zp(i, j-1)) - (yp(i, j), zp(i, j)), Farbe
    NEXT j
  NEXT i
    
  'Oberflaechenlinien der Funktion fuer y = const:
  FOR j = 0 TO ny
    FOR i = 1 TO nx
      LINE (yp(i-1, j), zp(i-1, j)) - (yp(i, j), zp(i, j)), Farbe
    NEXT i
  NEXT j
  
  IF zref THEN
    'Ebene z = 0 zeichnen:
    Rotation(x(0), y(0), 0, "z", gamma, xrot, yrot, zrot)
    Projektion(xrot, yrot, zrot, angp, yp01, zp01)
    Rotation(x(0), y(ny), 0, "z", gamma, xrot, yrot, zrot)
    Projektion(xrot, yrot, zrot, angp, yp02, zp02)
    Rotation(x(nx), y(ny), 0, "z", gamma, xrot, yrot, zrot)
    Projektion(xrot, yrot, zrot, angp, yp03, zp03)  
    Rotation(x(nx), y(0), 0, "z", gamma, xrot, yrot, zrot)
    Projektion(xrot, yrot, zrot, angp, yp04, zp04)
    LINE (yp01, zp01) - (yp02, zp02)
    LINE (yp02, zp02) - (yp03, zp03)
    LINE (yp03, zp03) - (yp04, zp04)
    LINE (yp04, zp04) - (yp01, zp01)
    'Eckpunkte der Ebene z = 0 mit den "Eckpunkten" der Funktion verbinden:
    LINE (yp01, zp01) - (yp(0, 0), zp(0, 0)),,,&HFF00
    LINE (yp02, zp02) - (yp(0, ny), zp(0, ny)),,,&HFF00
    LINE (yp03, zp03) - (yp(nx, ny), zp(nx, ny)),,,&HFF00
    LINE (yp04, zp04) - (yp(nx, 0), zp(nx, 0)),,,&HFF00
    'Eckpunkte der Ebene z = 0 beschriften:
    DRAW STRING (yp01, zp01), "(" + STR(x(0)) + ", " + STR(y(0)) + ")"
    DRAW STRING (yp02, zp02), "(" + STR(x(0)) + ", " + STR(y(ny)) + ")"
    DRAW STRING (yp03, zp03), "(" + STR(x(nx)) + ", " + STR(y(ny)) + ")"
    DRAW STRING (yp04, zp04), "(" + STR(x(nx)) + ", " + STR(y(0)) + ")"
  END IF
  
END SUB


SUB zrphiGraphik(r() AS SINGLE, phi() AS SINGLE, z() AS SINGLE, _
                 nphi AS INTEGER, nz AS INTEGER, gamma AS SINGLE, _
                 angp AS SINGLE, Farbe AS INTEGER)
  
' Funktion z(r, phi) in einem x-y-z-Koordinatensystem.
' Parameter:
' - Variable r von r(0, 0) bis r(nphi, nz)
' - Variable phi von phi(0) bis phi(nphi) im Bogenmass
' - Variable z von z(0) bis z(nz)
' - Farbe = Farbe der gezeichneten Linien
' - gamma = Drehwinkel um die z-Achse
' - angp = Projektionswinkel (siehe SUB Projektion).
' Die Funktion wird fuer konstante "Hoehenlinien" z(0) bis z(m) gezeichnet. 
' Die SUB eignet sich fuer Funktionen der Form z(r, phi) = f(r) * h(phi).
' Dann laesst sich im Hauptprogramm fuer jeden Wert z(j) und einen Wert phi(i)
' ein Wert r(i) = g(z(j)) / h(phi(i)) berechnen, wobei g die Umkehrfunktion
' von f ist.

  DIM AS SINGLE x(nphi, nz), y(nphi, nz), zneu(nphi, nz), yp(nphi, nz), _
                zp(nphi, nz), xrot, yrot, zrot
  DIM AS INTEGER i, j

  'Berechnung von z(r,phi) in kartesischen Koordinaten:
  FOR i = 0 TO nphi
    FOR j = 0 TO nz
      x(i, j) = r(i, j) * COS(phi(i))
      y(i, j) = r(i, j) * SIN(phi(i))
      zneu(i, j) = z(j)
    NEXT j
  NEXT i

  'Rotation und Projektion:
 FOR i = 0 TO nphi
    FOR j = 0 TO nz
         Rotation(x(i, j), y(i, j), zneu(i, j), "z", gamma, xrot, yrot, zrot)
         Projektion(xrot, yrot, zrot, angp, yp(i, j), zp(i, j))
    NEXT j
  NEXT i

  'Linien fuer z = const:
  FOR j = 0 TO nz
    FOR i = 1 TO nphi
      LINE (yp(i-1, j), zp(i-1, j)) - (yp(i, j), zp(i, j)), Farbe
    NEXT i
  NEXT j
  
  'Linien fuer phi = const:
  FOR i = 0 TO nphi
    FOR j = 1 TO nz
      LINE (yp(i, j-1), zp(i, j-1)) - (yp(i, j), zp(i, j)), Farbe
    NEXT j
  NEXT i

END SUB


SUB zrGraphik(r() AS SINGLE, z() AS SINGLE, n AS INTEGER, nphi AS INTEGER, _
              angp AS SINGLE, Farbe AS INTEGER)
  
  'Funktion z(r) in einem x-y-z-Koordinatensystem.
  'Parameter:
  '- Variable r von r(0) bis r(n)
  '- Variable z von z(0) bis z(n)
  '- nphi = Feldindex fuer phi() (wird in der SUB intern erzeugt)  
  '- Farbe = Farbe der gezeichneten Linien
  '- angp = Projektionswinkel (siehe SUB Projektion).
  'Die Funktion wird fuer konstante Radien r(0) bis r(n) gezeichnet.
  'Die SUB eignet sich fuer rotationssymmetrische Funktionen.
  
  DIM AS SINGLE phi(nphi), x(n, nphi), y(n, nphi), zneu(n, nphi), yp(n, nphi), _
                zp(n, nphi), pi
  DIM AS INTEGER i, j
  
  'Berechnung von z(r,phi) in kartesischen Koordinaten:
  pi = 3.141593
  FOR j = 0 TO nphi
    phi(j) = 2 * pi * j / nphi
    FOR i = 0 TO n
        x(i, j) = r(i) * COS(phi(j))
        y(i, j) = r(i) * SIN(phi(j))
        zneu(i, j) = z(i)
    NEXT i
  NEXT j

  'Projektion:
  FOR i = 0 TO n
    FOR j = 0 TO nphi
       Projektion(x(i, j), y(i, j), zneu(i, j), angp, yp(i, j), zp(i, j))
    NEXT j
  NEXT i

  'Linien fuer r = const:
  FOR i = 0 TO n
    FOR j = 1 TO nphi
      LINE (yp(i, j-1), zp(i, j-1)) - (yp(i, j), zp(i, j)), Farbe
    NEXT j
  NEXT i
    
  'Linien fuer phi = const:
  FOR j = 0 TO nphi
    FOR i = 1 TO n
      LINE (yp(i-1, j), zp(i-1, j)) - (yp(i, j), zp(i, j)), Farbe
    NEXT i
  NEXT j

END SUB

